# Version strings in HWS
#
# The main "Ruby" string is a legal version of our "Product Release
# Identification Standard":
#
# https://confluence.perforce.com:8443/display/EN/Product+Release+Identification+Standards
#
# A few notes:
#
# 1. While HWS does build for different OSes, it does not have an explicit
#    architecture. So we use "NOARCH".
#
# 2. The "Product ID" is currently "HWS".
#
# 3. We do include the date and changelist of the *tarball*.
#
# An official verison may be: HWS/NOARCH/2015.1/123456 (2015/08/17)
#
# A pre-release version will include information in the middle:
#
# HWS/NOARCH/2015.1.MAIN-TEST_ONLY/123456 (2015/08/17)
#
# For Ruby, the version string will be [YEAR].[REV].[CHANGELIST].
# Preleases will have a branch specifier: [YEAR].[REV].0.[branch][CHANGELIST].
#
# Ergo, a Ruby version string will look like 2015.1.0.main123456
#
# These strings are either defined in a file `./build_version.rb`.

module HelixWebServices
  VERSION = '2015.1.0.main'
end

file_path = File.absolute_path('../build_version.rb', __FILE__)
if File.exists?(file_path)
  require_relative './build_version'
else
  module HelixWebServices
    PRODUCT_ID = 'HWS/NOARCH/2015.1.MAIN/1'
  end
end
