require 'open-uri'
require 'helix_web_services_client/open_model'

class HelixWebServicesClient

  # Array of Branch objects stored in the system. Not all fields will be
  # filled out (e.g., view).
  def branches
    arr = execute_method_no_body(:get, hve_path('branches'))
    arr.map { |a| OpenModel.new(a) }
  end

  # Returns the branch instance indicated by the branch name (or model)
  def branch(branch)
    branch = OpenModel.new(branch).branch unless branch.is_a?(String)
    obj = execute_method_with_body(:get, hve_path("branches/#{URI.encode(branch)}"))
    OpenModel.new(obj)
  end

  # Creates a new branch in the system.
  def create_branch(branch)
    execute_method_with_body(:post, hve_path('branches'), branch)
    true
  end

  # Updates the branch specification.
  def update_branch(branch)
    branch = OpenModel.new(branch) unless branch.is_a?(OpenModel)
    execute_method_with_body(:patch, hve_path("branches/#{URI.encode(branch.branch)}"),
                             branch.marshal_dump)
    true
  end

  # Deletes the branch specification in the system.
  def delete_branch(branch)
    branch = OpenModel.new(branch).branch unless branch.is_a?(String)
    execute_method_no_body(:delete, hve_path("branches/#{URI.encode(branch)}"))
    true
  end
end
