require 'helix_web_services_client/open_model'
require 'open-uri'

class HelixWebServicesClient
  # Create a new client workspace specification for a Helix Sync project.
  #
  # Returns a simple object with a 'client' property.
  def create_helix_sync_client(project_id, device, root)
    data = {
        project: project_id,
        device: device,
        root: root
    }
    obj = execute_method_with_body(:post, '/helix_sync/v1/clients', nil, data)
    OpenModel.new(obj)
  end

  # Get a pending changelist number for the Helix Sync project.
  #
  # If the changelist already exists, it should just return that.
  #
  # IMPORTANT: The changelist should be updated to use your local client.
  # You won't be able to use or submit this changelist.
  #
  # @param project_id {String} The project ID.
  # @return A hash containing a `change` field
  def create_helix_sync_pending_changelist(project_id)
    obj = execute_method_no_body(:post, "/helix_sync/v1/changes/#{URI.encode(project_id)}")
    OpenModel.new(obj)
  end

  # Retrieve the latest changelist.
  #
  # If the change doesn't exist, you'll get a ResourceNotFound exception.
  def fetch_helix_sync_latest_changelist(project_id)
    obj = execute_method_no_body(:get, "/helix_sync/v1/changes/#{URI.encode(project_id)}")
    OpenModel.new(obj)
  end

  # Retrieve the pending changelist.
  #
  # If the change doesn't exist, you'll get a ResourceNotFound exception.
  def fetch_helix_sync_pending_changelist(project_id)
    obj = execute_method_no_body(:get, "/helix_sync/v1/changes/#{URI.encode(project_id)}/pending")
    OpenModel.new(obj)
  end

  # Submits the pending changelist for the helix sync project.
  #
  # Currently this assumes that all files are shelved and nothing's open on
  # the client. This is likely not what we want ... but it's a start.
  #
  # @param project_id [String]
  def submit_helix_sync_pending_change(project_id)
    obj = execute_method_no_body(:post, "/helix_sync/v1/changes/#{URI.encode(project_id)}/pending")
    OpenModel.new(obj)
  end
end
