require 'uri'

class HelixWebServicesClient

  # Lists the projects available on the server for the user.
  #
  # @param details [Boolean] Defaults to false, if true, output will be an array of project details.
  #                          If false, output will be an array of project IDs.
  #
  # @param extension [String] If set, will only return projects with the indicated extension ID.
  def projects(details: nil, extension: nil)
    query = {}
    query[:details] = details if details
    query[:extension] = extension if extension
    arr = execute_method_no_body(:get, '/projects/v1', query)
    arr = arr.map { |a| OpenModel.new(a) } if details
    arr
  end

  # Return the project details
  #
  # @param project_id [String] The project's String id
  def project(project_id)
    path = "/projects/v1/#{URI.encode(project_id)}"
    obj = execute_method_no_body(:get, path)
    OpenModel.new(obj)
  end
end