require 'open-uri'
require 'helix_web_services_client/open_model'

class HelixWebServicesClient
  # Returns an array of Server objects representing people in the system.
  def servers
    arr = execute_method_no_body(:get, hve_path('servers'))
    arr.map { |obj| OpenModel.new(obj) }
  end

  # Returns a HelixWebServicesClient::OpenModel for the login
  def server(server)
    server = server.ServerID if server.is_a?(OpenModel)
    obj = execute_method_no_body(:get, hve_path("servers/#{URI.encode(server)}"))
    OpenModel.new(obj)
  end

  # Creates a new server in the system based on the Server instance
  def create_server(server)
    server = OpenModel.new(server) unless server.is_a?(OpenModel)
    execute_method_with_body(:post, hve_path('servers'), server.marshal_dump)
  end

  def update_server(server)
    server = OpenModel.new(server) unless server.is_a?(OpenModel)
    execute_method_with_body(:patch,
                             hve_path("servers/#{URI.encode(server.ServerID)}"),
                             server.marshal_dump)
  end

  def delete_server(server)
    server = server.server_id if server.is_a?(OpenModel)
    execute_method_no_body(:delete, hve_path("servers/#{URI.encode(server)}"))
  end
end
