require 'open-uri'
require 'helix_web_services_client/open_model'

class HelixWebServicesClient
  # Returns an array of User objects representing people in the system.
  def users
    arr = execute_method_no_body(:get, hve_path('users'))
    arr.map { |obj| OpenModel.new(obj) }
  end

  # Returns a HelixWebServicesClient::OpenModel for the login
  def user_details(user)
    user = user.user if user.is_a?(OpenModel)
    obj = execute_method_no_body(:get, hve_path("users/#{URI.encode(user)}"))
    OpenModel.new(obj)
  end

  # Creates a new user in the system based on the User instance
  def create_user(user)
    user = OpenModel.new(user) unless user.is_a?(OpenModel)
    execute_method_with_body(:post, hve_path('users'), user.marshal_dump)
  end

  def update_user(user)
    user = OpenModel.new(user) unless user.is_a?(OpenModel)
    execute_method_with_body(:patch, hve_path("users/#{URI.encode(user.user)}"),
                             user.marshal_dump)
  end

  def delete_user(user)
    user = user.user if user.is_a?(OpenModel)
    execute_method_no_body(:delete, hve_path("users/#{URI.encode(user)}"))
  end
end
